function [Q R]=ReducGiv(R, col_1, col_2)

%
% [Q R] = ReducGiv(R,col_1,col_2) computes the QR decomposition of R after
% columns col_1 and col_2 are interchanged, where col_1 < col_2.
%
% Input arguments:
%    R     --- n by n upper triangular real matrix
%    col_1 --- Integer, column number of the leftmost column to be interchanged 
%    col_2 --- Integer, column number of the rightmost column to be interchanged	
%
% Output arguments:
%    Q --- n by n orthogonal matrix. It's the product of the Givens rotations to bring
%          R back to an upper triangular matrix after the column
%          interchange. 
%    R --- n by n real upper triangular matrix.  It's the matrix R after
%          the column interchange and the Givens rotations. 
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0,  May 2007.



if col_2 <= col_1,
   display('Error in input arguments');    
end

m = size(R,1);
Q = eye(m);
G = eye(m);
%col_1 and col_2 are switched
R(:,[col_1 col_2])= R(:,[col_2 col_1]);

% Givens rotations to eliminate the last col_2 - col_1 elements of column col_1 
for i = col_2:-1:col_1+1
J=JGiv(R(i-1, col_1),R(i,col_1));
R([i-1 i],:) = J*R([i-1 i],:); 
G([i-1 i],[i-1 i]) = J;
Q = G * Q; 
G = eye(m);
end

% Givens rotations to eliminate the subdiagonal entries of R 
for i= col_1+2:1:col_2
J = JGiv(R(i-1,i-1), R(i,i-1));
R([i-1 i],:) = J*R([i-1 i],:); 
G([i-1 i],[i-1 i])= J;
Q = G * Q;
G = eye(m);
end
