function s=compute_sigma(R,y,l,u)
%
%  s = compute_sigma(R,y,l,u) produces the sigma values needed by the search
%  algorithm.
%       
%  Input arguments:
%    R ---- n by n real nonsingular upper triangular matrix
%    y ---- n-dimensional real vector
%    l ---- n-dimensional lower-bound integer vector
%    u ---- n-dimensional upper-bound integer vector
%
% Output argument:
%    s ---- n-dimensional real vector containing the sigma values.     
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.


n = length(y);
s = zeros(n,1);
d = zeros(n,1);

 %  The formula for d(k) is:
 % d(k) = min    { [y(k) - Sum_j=k_to_n{ min[R(k,j)*l(k),R(k,j)*u(k)] } ]^2 , 
 %                 [y(k) - Sum_j=k_to_n{ max[R(k,j)*l(k),R(k,j)*u(k)] } ]^2 
 %               }
 % If the two arguments of the min statement above have different signs (if the square is removed), then d(k) = 0. 

for k = 1:1:n
   
    min_sum = 0;
	max_sum =0;
    
    % min_sum and max_sum are the two sums in the definition of d(k)
    for j = k:1:n
        
        % TODO replace j's by k's in l and u
        min_sum = min_sum + min(R(k,j)*l(j), R(k,j)*u(j));
        max_sum = max_sum + max(R(k,j)*l(j), R(k,j)*u(j));  
    end

    if sign(y(k) - min_sum) == sign(y(k) - max_sum)
        d(k) = min((y(k) - min_sum)^2, (y(k) - max_sum)^2);            
    else % by definition, if the signs of (y(k) - min_sum) and (y(k)-max_sum) are different, then d(k)=0
        d(k) = 0;
    end
end

s(1) = 0; % by definition

 % The formula for s(k) is:
 % s(k) = sum_i=1_to_k-1 {d(i)}
for k = 2:1:n
     s(k) = sum(d(1:k-1));
end

