function [ck zk lbounds ubounds delta]=compute_zk(R,y,z,l,u,k)
%
% [ck zk lbounds ubounds delta]=compute_zk(R,y,z,l,u,k) computes the
% c(k), z(k), lbounds(k), ubounds(k), and delta(k) values necessary 
% for the search algorithm.
%                           
% Input arguments:
%    R ---- n by n real nonsingular upper triangular matrix
%    y ---- n-dimensional real vector
%    z ---- n-dimensional integer vector.  Its final value is the optimal
%           solution to the upper triangular integer least squares problem. 
%    l ---- n-dimensional lower-bound integer vector
%    u ---- n-dimensional upper-bound integer vector
%    k ---- integer representing the current level in the search
%
% Output arguments:
%    ck   --- real number  
%    zk   --- nearest integer to ck within the constraints
% lbounds --- flag whose value is 1 when zk is below the lower bound l(k). 
%             Otherwise, its value is 0.
% ubounds --- flag whose value is 1 when zk is larger than the upper bound u(k). 
%             Otherwise, its value is 0.
%   delta --- Integer number used for enumeration of new points     
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.


n = size(y,1);
ck = (y(k) - R(k,k+1:n) * z(k+1:n))/R(k,k); % by definition  

zk = nearInterval(l(k),u(k),ck,1);

lbounds = 0;
ubounds = 0;

% zk is within the constraints since we have used the nearInterval
% routine.
if zk == l(k)
    lbounds = 1;
	delta = 1;
elseif zk == u(k)
    ubounds = 1;
	delta = -1;
else % no boundary of the constraints is reached
	delta = sign(ck - zk);
end


