function x=EILS(A,y,a)
%
% x = EILS(A,y,a) produces the solution to the ellipsoid-constrained 
% integer least squares (EILS) problem 
% min_x||y-Ax||^2,  where norm(A*x) <=  a 
%               
% Input arguments:
%    A ---- m by n real matrix with full column rank
%    y ---- m-dimensional real vector
%    a ---- real number.  It's the ellipsoid-constraint
% 
% Output argument:
%    x ---- n-dimensional integer vector.  It's the solution to the EILS 
%           problem.  
%    
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, June 2007.

if nargin < 3 
    error('Missing input arguments')
end

[m,n] = size(A);
if m < n || m ~= size(y,1) || size(y,2) ~= 1   
    error('Input arguments have a matrix dimension error')
end

if rank(A) < n 
	error('Matrix is rank defficient')
end

% LLL-QRZ factorization. It's the reduction phase
[R,Z,y] = reductionLLL(A,y);
y = y(1:n);

% The diagonal entries of R should be positive
[R y]=positive_diag(R,y);

% Search phase 
zh=search_eils(R,y,inf,a); % the solution to the upper triangular EILS problem
x=Z*zh; % the solution to the general EILS problem
 
