function z=nearInterval(l,u,c,choice)
%
%  z = nearInterval(l,u,c,choice) computes the nearest or the 
%  second nearest integer on the interval  [l,u] to c.  
% 
% Input arguments:
%    l   --- integer representing the lower bound on the interval
%    u   --- integer representing the upper bound on the interval
%    c   --- real number upon which we must find a near integer.
% choice --- Integer flag.  If it's value is 2, it indicates we 
%            search for the second nearest integer on the interval. 
%            Otherwise, we search for the nearest integer on the interval. 
% 
% Output arguments:
%    z ---  nearest or second nearest integer to c on the interval.
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.


% Check input arguments
if l > u
    tmp=u;
    u = l;
    l = tmp;
end


if choice == 2  % find the second nearest integer to c in the interval
lower = 0;
upper = 1;
else            % find the nearest integer to c in the interval
lower = 1;
upper = 0;
end


if l <= c && c <= u  % c is inside the interval
    c_floor = floor(c);
    if c - c_floor ~= 0.5,        % c does not have 0.5 in decimal
       if c_floor == round(c)
           z = c_floor + upper;
       else
           z = c_floor+lower;
       end
  
    else
        z = round(c);
        if choice == 2           
           z = z + -1*sign(c);       
        end
    end  
    
else
    if c < l,
        z = l + upper;
    else
        % case c > u
       z = u - upper;
    end
end


