function [R Z y]=reductionLLL(A,y)

% [R,Z,y] = reduction(A,y) computes the LLL-QRZ factorization:
%           Q'*A*Z = [R; 0] and computes Q'*y. The orthogonal matrix Q 
%           is not produced. Its goal is to reduce a general integer
%           least squares problem to an upper triangular one.
%
% Input arguments:
%    A --- m by n real matrix with full column rank
%    y --- m-dimensional real vector to be transformed to Q'y
%
% Output arguments:
%    R --- n by n LLL-reduced upper triangular matrix
%    Z --- n by n unimodular matrix, i.e., an integer matrix with |det(Z)|=1
%    y --- m-vector transformed from the input y by Q', i.e., y := Q'*y

% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, June 2007.


[m n] = size(A);
[Q R]=qr(A);
Q=Q(1:m,1:n);
y = Q'*y;

R=R(1:n,1:n);
Z = eye(n);
k = 2;

while k <= n
    
   for i = k-1:-1:1
       % Perform off-diagonal size reduction for R(:,k)
          alpha = round(R(i,k)/R(i,i));
        if alpha ~= 0
            R(1:i,k) = R(1:i,k) - alpha*R(1:i,i);
            Z(:,k) = Z(:,k) - alpha*Z(:,i);
        end
   end
   
   if R(k-1,k-1)^2 > R(k-1,k)^2 + R(k,k)^2;  
       % Permute columns k-1 and k of R, and bring it back to an upper
       % triangular matrix
        [Q R]=ReducGiv(R,k-1,k);
        Z(:,[k-1 k]) = Z(:,[k k-1]);
        y = Q*y;
        if k > 2
            k = k-1;
        end
   else
       k = k+1;
   end
end